@@ -220,8 +220,8 @@ setMethod("plot", signature(x = "GeDS"), function(x, f = NULL, which, DEV = FALS
     type <- match.arg(type)
     
     # Format independent variable name for plotting
-    var_labels <- attr(x$terms, "variables")  # Full list of variables
-    xname <- as.character(var_labels)[3]  # Exclude response variable
+    var_labels <- attr(x$terms, "variables") 
+    xname <- as.character(var_labels)[3]
     # Extract dependent variable name for plotting
     yname <- rownames(attr(x$terms,"factors"))[1]
     # Use provided labels or defaults
@@ -688,6 +688,567 @@ setMethod("plot", signature(x = "GeDS"), function(x, f = NULL, which, DEV = FALS
 }
 )
 
+plot.GeDS <- function(x, f = NULL, which, DEV = FALSE, ask = FALSE,
+                      main, legend.pos = "topright", legend.text = NULL,
+                      new.window = FALSE, wait = 0.5,
+                      n = 3L, type = c("none", "Polygon", "NCI", "ACI"), ...)
+{
+  results <- list()
+  results$terms <- x$terms
+  
+  # Position of the legend within the panel
+  draw.legend <- !(is.na(legend.pos))
+  
+  # Check length of DEV/ask/new.window/wait
+  if(length(DEV)!= 1 || length(ask)!= 1 || length(new.window)!= 1 || length(wait)!= 1 )
+    stop("Please, check the length of the parameters")
+  # Logical variable specifying whether a plot representing the deviance at each iteration of stage A should be produced or not
+  DEV <- as.logical(DEV)
+  # Logical variable specifying whether the user should be prompted before changing the plot page
+  ask <- as.logical(ask)
+  # Logical variable specifying whether the plot should be shown in a new window or in the active one
+  new.window <- as.logical(new.window)
+  # Time, in seconds, the system should wait before plotting a new page
+  wait <- as.numeric(wait)
+  
+  if (ask) {
+    # Prompt before new page
+    oask <- devAskNewPage(TRUE)
+    # Restore the original setting
+    on.exit(devAskNewPage(oask))
+    # Since the plot waits for user input, set the sleep/wait time to 0
+    slp <- 0
+  } else {
+    slp <- wait
+  }
+  
+  # Extract arguments
+  X <- x$Args$X; Y <- x$Args$Y; Z <- x$Args$Z
+  weights <- x$Args$weights; q <- x$Args$q; offset <- x$Args$offset 
+  
+  # Check if order is correctly set
+  n <- as.integer(n)
+  if(!(n %in% 2L:4L)) {
+    n <- 3L
+    warning("'n' incorrectly specified. Set to 3.")
+  }
+  
+  # Set toprint
+  if (n == 2L) {
+    toprint = "Linear"
+  } else if (n==3L) {
+    toprint = "Quadratic"
+  } else if (n==4L) {
+    toprint = "Cubic"
+  }
+  
+  maxim <- nrow(x$Stored) # number of iterations in stage A
+  others <- list(...) # other arguments passed to the function
+  
+  # Data arguments
+  col_data <- if (is.null(others$col)) "black" else others$col
+  pch_data <- if (is.null(others$pch)) 1 else others$pch
+  
+  ########################
+  ## 1. Univariate GeDS ##
+  ########################
+  if (x$Type == "LM - Univ" || x$Type == "GLM - Univ") {
+    
+    # Set plot color, default to "red" if not specified in additional arguments
+    col_lines <- if ("col_lines" %in% names(others)) others$col_lines else "red"
+    others$col_lines = NULL # remove 'col_lines' from additional arguments to prevent conflicts
+    
+    # Set default iteration(s) to plot to k + 1 if not specified
+    if (missing(which)) which <- x$Nintknots + 1
+    # If which == "all", plot all stage A iterations
+    if (length(which) == 1 && which == "all") which <- 1:(maxim)
+    # Independent variable extremes
+    extr <- x$Args$extr
+    
+    # Validate ("Polygon", "NCI" or "ACI") and set plot type
+    type <- match.arg(type)
+    
+    # Format independent variable name for plotting
+    var_labels <- attr(x$terms, "variables")  # Full list of variables
+    xname <- as.character(var_labels)[3]  # Exclude response variable
+    # Extract dependent variable name for plotting
+    yname <- rownames(attr(x$terms,"factors"))[1]
+    # Use provided labels or defaults
+    xlab <- if (!is.null(others$xlab)) others$xlab else if ( !missing(main) && main=="detail" ) "" else xname
+    ylab <- if (!is.null(others$ylab)) others$ylab else yname
+    
+    # Set y-axis limits
+    yylim <- range(c(Y, x$Linear.Fit$Predicted)) + 0.05 * c(-1, 1) *  range(c(Y, x$Linear.Fit$Predicted))
+    ylim <- if (!is.null(others$ylim)) others$ylim else yylim
+    
+    # Determine the maximum number of iterations
+    maxim <- nrow(x$Stored)
+    
+    # Validate 'which' parameter
+    if ((!is.numeric(which) && !is.null(which)) || any(which < 1) || any(which > maxim)) {
+      stop(sprintf("'which' must be between 1 and %d", maxim), domain = NA)
+    }
+    if (!is.null(which)) {
+      which <- as.integer(which)
+      last <- max(which)
+    } else {
+      last <-  0
+    }
+    # Warn if plotting iterations discarded by the algorithm
+    if (any(which > maxim - q)) {
+      warning("Plotting also iterations discarded by the algorithm")
+    }
+    
+    # If generating function is also to be plotted
+    if (!is.null(f)) {
+      if (!is.function(f)) {
+        stop("f must be a function")
+      }
+      # Check the number of arguments that f takes
+      num_args <- length(formals(f))
+      # If the function takes more than one argument, display a message
+      if (num_args > 1) {
+        stop("The function f should take only one argument. Instead of f(X, Y), you should provide f(X), with X having two columns.")
+      }
+      
+      f_legend <- "f"; f_lty <- 1; f_col <- "black"; f_pch <- NA; f_lwd <- 2
+      
+    } else {
+      f_legend <- f_lty <- f_col <- f_pch <- f_lwd <- NA
+    }
+    
+    ###############################
+    ## 1. Univariate NGeDS/GGeDS ##
+    ###############################
+    if (x$Type == "GLM - Univ") {
+      # Extract GLM family from model arguments
+      family <- x$Args$family
+      # Restrict plotting to linear spline in case which does not correspond to the final model's iteration
+      if (n != 2L && which != maxim - q) {
+        which <- maxim - q
+        warning("Stage A iterations can be plotted only for the linear spline")
+      }
+    }
+    
+    # Loop over specified iterations for plotting
+    for (i in which) {
+      
+      # Logical variable specifying whether the plot should be shown in a new window or in the active one
+      if(new.window) dev.new()
+      if(slp > 0) {
+        Sys.sleep(slp)
+      }
+      # Set main plot title
+      main0 <- if (missing(main)) {
+        paste0(i-1, " internal knots")
+      } else if (main == "detail") {
+        ""
+      } else {
+        main
+      }
+      # Plot
+      plot(X, Y, main = main0, ...)
+      if (!is.null(f)) lines(X, f(X), col = "black", lwd = 2)
+      
+      # Obtain stage A knots and perform spline regression
+      ik <- na.omit(x$Stored[i,-c(1,2,(i+2),(i+3))])
+      # Stage B.1 (averaging knot location)
+      int.knt <- if (i > 1) makenewknots(ik, n) else NULL
+      
+      ###########################
+      ## 1.1. Univariate NGeDS ##
+      ###########################
+      if (x$Type == "LM - Univ") {
+        # Stage B.2
+        temp <- SplineReg_LM(X = X, Y = Y, Z = Z, offset = offset, weights = weights, extr = extr,
+                             InterKnots = int.knt, n = n)
+        
+        ###########################
+        ## 1.2. Univariate GGeDS ##
+        ###########################
+      } else if (x$Type == "GLM - Univ") {
+        # Stage B.2
+        temp <- SplineReg_GLM(X = X, Y = Y, Z = Z, offset = offset, weights = weights, extr = extr,
+                              InterKnots = int.knt, n = n, family = family, mustart = x$Linear.Fit$Predicted)
+      }
+      
+      # Update results with predicted values
+      results$X <- X
+      results$predicted <- temp$Predicted
+      
+      # Plot the spline
+      lines(X, temp$Predicted, col = col_lines, lwd = 2)
+      
+      # Add vertical lines for knots
+      if (length(int.knt) < 20) {
+        for(ik in c(int.knt, extr)) {
+          abline(v = ik, col = "gray", lty = 2)
+        }
+      } else {
+        rug(c(int.knt, extr))
+      }
+      # Detailed knots vector
+      if (!missing(main) && main == "detail") {
+        # knot
+        knot_values <- round(knots(x, n=2), 2)
+        intknt_text <- if (!is.null(get_internal_knots(knot_values, 2))) {
+          paste0(", ", get_internal_knots(knot_values, 2))
+        } else {
+          ""
+        }
+        knt_text <- paste0(
+          "a = ",
+          knot_values[1],
+          paste(intknt_text, collapse = ""),
+          ", b = ",
+          knot_values[length(knot_values)]
+        )
+        if (length(knt_text) > 18) {
+          knt_text1 <- paste(knt_text[1:round(length(knt_text) / 2)], collapse = ", ")
+          knt_text2 <- paste(knt_text[(round(length(knt_text) / 2) + 1):length(knt_text)], collapse = ", ")
+          
+          knots_text1 <- bquote(italic(bold(t)[1 * ";" * k[1] * "," * 2]) == italic(.(paste0("{", knt_text1))))
+          knots_text2 <- bquote(italic(.(paste0(knt_text2, "}"))))
+        } else {
+          knots_text1 <- bquote(italic(bold(t)[1 * ";" * k[1] * "," * 2]) == italic(.(paste0("{", knt_text,"}"))))
+          knots_text2 <- ""
+        }
+        
+        mtext(knots_text1, side = 1, line = 3, cex = 0.9)
+        mtext(knots_text2, side = 1, line = 4, cex = 0.9)
+      }
+      
+      ## Each branch now adds specific elements to the plot based on the selected type
+      # 1) Polygon
+      if (type == "Polygon") {
+        results$Polykn <- temp$Poly$Kn
+        results$Polyth <- if (x$Type == "LM - Univ") temp$Poly$Thetas else if (x$Type == "GLM - Univ") family$linkinv(temp$Poly$Thetas)
+        
+        lines(results$Polykn, results$Polyth, col = "blue", lty = 2)
+        points(results$Polykn, results$Polyth, col = "blue")
+        
+        if (missing(legend.text)) legend.text <- c("Data", toprint, "Polygon", f_legend)
+        
+        if(draw.legend) legend(legend.pos, legend.text, lty = c(NA, 1, 2, f_lty),
+                               col = c(col_data, col_lines, "blue", f_col),
+                               pch = c(pch_data, NA, 1, f_pch),
+                               lwd = c(NA, 1, 1, f_lwd), bty = "n")
+        
+        ## NCI/ACI
+      } else if(type == "NCI" || type == "ACI") {
+        if (missing(legend.text)) legend.text <- c("Data", toprint, "CI", f_legend)
+        
+        if (x$Type == "LM - Univ") {
+          
+          # 2) Normal Confidence Intervals
+          if (type == "NCI") {
+            lines(X,temp$NCI$Upp, col = col_lines, lwd = 2, lty = 2)
+            lines(X,temp$NCI$Low, col = col_lines, lwd = 2, lty = 2)
+            
+            results$CIupp <- temp$NCI$Upp
+            results$CIlow <- temp$NCI$Low
+            
+            if(draw.legend) legend(legend.pos, legend.text, lty = c(NA, 1, 2, f_lty),
+                                   col = c(col_data, col_lines, col_lines, f_col),
+                                   pch = c(pch_data, NA, NA, f_pch),
+                                   lwd = c(NA, 1, 1, f_lwd), bty = "n")
+            
+            # 3) Asymptotic Confidence Intervals
+          } else if (type == "ACI") {
+            lines(X,temp$ACI$Upp, col = col_lines, lwd = 2, lty = 2)
+            lines(X,temp$ACI$Low, col = col_lines, lwd = 2, lty = 2)
+            results$CIupp <- temp$ACI$Upp
+            results$CIlow <- temp$ACI$Low
+            
+            if(draw.legend) legend(legend.pos, legend.text, lty = c(NA, 1, 2, f_lty),
+                                   col = c(col_data, col_lines, col_lines, f_col),
+                                   pch = c(pch_data, NA, NA, f_pch),
+                                   lwd = c(NA, 1, 1, f_lwd), bty = "n")
+          }
+          
+        } else if (x$Type == "GLM - Univ") {
+          
+          # 2) Normal Confidence Intervals
+          if (type == "NCI") {
+            yy <- Y
+            xx <- if (n != 2L) {
+              temp$Basis
+            } else {
+              x$Linear.Fit$Basis
+            }
+            
+            matrice <- splineDesign(knots = sort(c(int.knt,rep(extr,2))), derivs = rep(0,length(X)),
+                                    x = X, ord = n, outer.ok = TRUE)
+            
+            temp_nci <- glm(yy ~ -1+xx, offset = offset, weights = weights, family = family, start = temp$Thetas)
+            pred <- predict.glm(temp_nci, newdata = data.frame(xx = matrice, offset = 0), se.fit = T, type = "response")
+            CIupp <- pred$fit + qnorm(.975)*pred$se.fit
+            CIlow <- pred$fit - qnorm(.975)*pred$se.fit
+            
+            lines(X, CIupp, col = "black", lwd = 2, lty = 2)
+            lines(X, CIlow, col = "black", lwd = 2, lty = 2)
+            
+            results$CIupp <- CIupp
+            results$CIlow <- CIlow
+            
+            if(draw.legend) legend(legend.pos, c("Data", toprint, "CI", f_legend), lty = c(NA, 1, 2, f_lty),
+                                   col = c(col_data, col_lines, "black", f_col), pch = c(pch_data, NA, NA, f_pch),
+                                   lwd = c(NA, 1, 1, f_lwd), bty = "n")
+            
+            # 3) Asymptotic Confidence Intervals
+          } else if (type == "ACI") {
+            CI <- confint.GeDS(object = x, n = n)
+            CIupp <- CI[,2]
+            CIlow <- CI[,1]
+            results$CIupp <- CIupp
+            results$CIlow <- CIlow
+            
+            lines(X, CIupp, col = col_lines, lwd = 2, lty = 2)
+            lines(X, CIlow, col = col_lines, lwd = 2, lty = 2)
+            
+            if(draw.legend) legend(legend.pos, c("Data", toprint, "CI", f_legend), lty = c(NA, 1, 2, f_lty),
+                                   col = c(col_data, col, "black", f_col), pch = c(pch_data, NA, NA, f_pch),
+                                   lwd = c(NA, 1, 1, f_lwd), bty = "n")
+          }
+        }
+        
+      } else {
+        if (missing(legend.text)) legend.text <- c("Data", toprint, f_legend)
+        if (draw.legend) legend(legend.pos, legend.text, lty = c(NA, 1, f_lty),
+                                col = c(col_data, col_lines, f_col), pch = c(pch_data, NA, f_pch),
+                                lwd = c(NA, 1, f_lwd), bty = "n")
+      }
+    }
+    
+    ## If DEV = TRUE, produce plot representing the deviance at each iteration of stage A
+    if (x$Type == "LM - Univ") {
+      print <- "RSS"
+    } else if (x$Type == "GLM - Univ") {
+      print <- "DEV"
+    }
+    
+    if (DEV) {
+      # i. RSS/DEV
+      main0 <- if(missing(main)) print else main
+      plot(seq(0, x$iters-1), x$RSS/length(X), main = main0, xlab = "Internal knots", ylab = "", ...)
+      rug(seq(1, x$iters-1 - q))
+      if (slp > 0) {
+        Sys.sleep(slp)
+      }
+      # ii. \sqrt{RSS} / \sqrt{DEV}
+      main0 <- if(missing(main)) bquote(sqrt(.(print))) else main
+      plot(seq(0, x$iters-1), (x$RSS/length(X))^.5, main = main0, xlab = "Knots", ylab = "", ...)
+      rug(seq(1, x$iters-1 - q))
+      if(slp > 0) {
+        Sys.sleep(slp)
+      }
+      # iii. \phi
+      main0 <- if(missing(main)) expression(phi) else main
+      plot(seq(q, x$iters-1), (x$RSS[(1+q):x$iters]/x$RSS[(1):(x$iters-q)]), main = main0, xlab="Knots", ylab = "", ...)
+      rug(seq(2, x$iters-1 - q))
+      if(slp > 0) {
+        Sys.sleep(slp)
+      }
+      # iv. \sqrt{\phi}
+      main0 <- if(missing(main)) expression(sqrt(phi)) else main
+      plot(seq(q, x$iters-1), (x$RSS[(1+q):x$iters]/x$RSS[(1):(x$iters-q)])^.5, main = main0, xlab="Knots", ylab = "", ...)
+      rug(seq(2, x$iters-1 - q))
+      if(slp > 0) {
+        Sys.sleep(slp)
+      }
+    }
+    
+    ##############################
+    ## 2. Bivariate NGeDS/GGeDS ##
+    ##############################
+  } else if (x$Type == "LM - Biv" || x$Type == "GLM - Biv") {
+    
+    if(n == 2L) {
+      obj <- x$Linear.Fit
+    } else if (n == 3L) {
+      obj <- x$Quadratic.Fit
+    } else if (n == 4L) {
+      obj <- x$Cubic.Fit
+    }
+    
+    # Extract parametric component of the predictor model + X, Y extremes
+    W <- x$Args$W; Xextr <- x$Args$Xextr; Yextr <- x$Args$Yextr
+    
+    # Default labels
+    var_labels <- as.character(attr(x$terms, "variables"))[3]
+    var_labels <- regmatches(var_labels, regexec("f\\(([^,]+), ([^)]+)\\)", var_labels))
+    xname <- var_labels[[1]][2]; yname <- var_labels[[1]][3]
+    xlab <- if (!is.null(others$xlab)) others$xlab else xname
+    ylab <- if (!is.null(others$ylab)) others$ylab else yname
+    zlab <- if (!is.null(others$zlab)) {
+      others$zlab
+      } else if (!is.null(f)) {
+        var_labels[[1]][1]
+        } else {
+          gsub("f", "f_hat", var_labels[[1]][1], fixed = TRUE)
+        }
+    
+    # Ensure they are removed from the 'others' list before passing '...'
+    others <- others[!names(others) %in% c("xlab", "ylab", "zlab")]
+    
+    if (!is.null(f)) {
+      if (!is.function(f)) {
+        stop("f must be a function")
+      }
+      # Check the number of arguments that f takes
+      num_args <- length(formals(f))
+      # If the function takes more than one argument, display a message
+      if (num_args > 1) {
+        stop("The function f should take only one argument. Instead of f(X, Y), you should provide f(X), with X having two columns.")
+      }
+      
+      # Create a grid of x, y values
+      seq_valX <- seq(Xextr[1], Xextr[2], length.out = round(sqrt(length(X))))
+      seq_valY <- seq(Xextr[1], Xextr[2], length.out = round(sqrt(length(Y))))
+      grid_val <- expand.grid(x = seq_valX, y = seq_valY)
+      # Compute z for each x, y in the grid
+      f_XY_val <- matrix(f(grid_val), nrow = round(sqrt(length(X))))
+      # Title based on the number of internal knots in X and Y
+      if(missing(main)) {
+        main <- paste0(x$Nintknots$X, " internal knots in X and ",
+                       x$Nintknots$Y ," internal knots in Y" )
+      }
+      
+      # Plot
+      persp3D(x = seq_valX, y = seq_valY, z = f_XY_val, phi = 25, theta = 50,
+              zlim = range(f_XY_val), ticktype = "detailed", expand = 0.5,
+              colkey = FALSE, border = "black",
+              xlab = xlab, ylab = ylab, zlab = zlab)
+      # Add the data and predicted points to the plot
+      points3D(x = X, y = Y, z = Z, col = "black", pch = 19, add = TRUE)
+      points3D(x = X, y = Y, z = obj$Predicted, col = "red", pch = 19, add = TRUE)
+      legend("topright",
+             legend = c("Data", "Quadratic Fit"),
+             col = c("black", "red"),
+             pch = 19,
+             bg = 'white',
+             bty = "n")
+      
+    } else {
+      
+      df <- data.frame(X,Y)
+      
+      # Create a sequence of sqrt(obs) evenly spaced numbers over the range of X/Y
+      newX <- seq(from = Xextr[1], to = Xextr[2], length.out = round(sqrt(length(X))))
+      newY <- seq(from = Yextr[1], to = Yextr[2], length.out = round(sqrt(length(Y))))
+      
+      # Create a grid data frame from all combinations of newX and newY
+      grid.data <- expand.grid(newX, newY)
+      
+      # Generate spline basis matrix for X and Y dimensions using object knots and given order
+      basisMatrixX <- splineDesign(knots = obj$Xknots, derivs = rep(0,length(grid.data[,1])),
+                                   x = grid.data[,1], ord = n, outer.ok = T)
+      basisMatrixY <- splineDesign(knots = obj$Yknots, derivs = rep(0,length(grid.data[,2])),
+                                   x = grid.data[,2], ord = n, outer.ok = T)
+      # Calculate the tensor product of X and Y spline matrices to create a bivariate spline basis
+      basisMatrixBiv <- tensorProd(basisMatrixX, basisMatrixY)
+      # Multiply the bivariate spline basis by model coefficients to get fitted values
+      f_XY_hat_val <- basisMatrixBiv %*% obj$Theta[1:dim(basisMatrixBiv)[2]]
+      # Reshape the fitted values to a square matrix for plotting
+      f_XY_hat_val <- matrix(f_XY_hat_val, nrow = length(newX) )
+      if (x$Type == "GLM - Biv") f_XY_hat_val <- x$Args$family$linkinv(f_XY_hat_val)
+      # Title based on the number of internal knots in X and Y
+      if(missing(main)) {
+        main <- paste0(x$Nintknots$X, " internal knots in X and ",
+                       x$Nintknots$Y ," internal knots in Y" )
+      }
+      
+      # Plot the perspective 3D surface defined by newX, newY, and the fitted values
+      persp3D(x = newX, y = newY, z = f_XY_hat_val, phi = 25, theta = 50,
+              zlim = range(f_XY_hat_val), ticktype = "detailed", expand = 0.7,
+              colkey = FALSE, border = "black",
+              xlab = xlab, ylab = ylab, zlab = zlab)
+      
+      if (main == "detail") {
+        # KnotsX
+        knotsX_values <- round(knots(x, n=2)$Xk,2)
+        intkntX_text <- if (!is.null(get_internal_knots(knotsX_values,2))) {
+          paste0(", ", get_internal_knots(knotsX_values,2))
+        } else {
+          ""
+        }
+        kntX_text <- paste0(
+          "a = ",
+          knotsX_values[1],
+          paste(intkntX_text, collapse = ""),
+          ", b = ",
+          knotsX_values[length(knotsX_values)]
+        )
+        if (length(kntX_text) > 18) {
+          kntX_text1 <- paste(kntX_text[1:round(length(kntX_text) / 2)], collapse = ", ")
+          kntX_text2 <- paste(kntX_text[(round(length(kntX_text) / 2) + 1):length(kntX_text)], collapse = ", ")
+          
+          knotsX_text1 <- bquote(italic(bold(t)[1 * ";" * k[1] * "," * 2]) == italic(.(paste0("{", kntX_text1))))
+          knotsX_text2 <- bquote(italic(.(paste0(knt_text2, "}"))))
+        } else {
+          knotsX_text1 <- bquote(italic(bold(t)[1 * ";" * k[1] * "," * 2]) == italic(.(paste0("{", kntX_text,"}"))))
+          knotsX_text2 <- ""
+        }
+        
+        mtext(knotsX_text1, side = 3, line = 2, adj = 0.5, cex = 0.7)
+        mtext(knotsX_text2, side = 3, line = 1, adj = 0.5, cex = 0.7)
+        
+        # Knots Y
+        knotsY_values <- round(knots(x, n=2)$Yk,2)
+        intkntY_text <- if (!is.null(get_internal_knots(knotsY_values,2))) {
+          paste0(", ", get_internal_knots(knotsY_values,2))
+        } else {
+          ""
+        }
+        kntY_text <- paste0(
+          "a = ",
+          knotsY_values[1],
+          paste(intkntY_text, collapse = ""),
+          ", b = ",
+          knotsY_values[length(knotsY_values)]
+        )
+        
+        if (length(kntY_text) > 18) {
+          kntY_text1 <- paste(kntY_text[1:round(length(kntY_text) / 2)], collapse = ", ")
+          kntY_text2 <- paste(kntY_text[(round(length(kntY_text) / 2) + 1):length(kntY_text)], collapse = ", ")
+          
+          knotsY_text1 <- bquote(italic(bold(t)[1 * ";" * k[2] * "," * 2]) == italic(.(paste0("{", kntY_text1))))
+          knotsY_text2 <- bquote(italic(.(paste0(knt_text2, "}"))))
+        } else {
+          knotsY_text1 <- bquote(italic(bold(t)[2 * ";" * k[2] * "," * 2]) == italic(.(paste0("{", kntY_text,"}"))))
+          knotsY_text2 <- ""
+        }
+        mtext(knotsY_text1, side = 3, line = 0, adj = 0.5, cex = 0.7)
+        mtext(knotsY_text2, side = 3, line = -1, adj = 0.5, cex = 0.7)
+      }
+      
+      
+      # Adjust Z by subtracting the parametric component of the predictor model if it exists
+      if (!is.null(W)) {
+        Z <- Z - W %*% x$Linear.Fit$Theta[-(1:dim(basisMatrixBiv)[2])]
+      }
+      # Identify points where Z is greater/smaller than the model predicted value
+      tmp <- (obj$Predicted - Z > 0)
+      points3D(x = X[!tmp], y = Y[!tmp], z = Z[!tmp], col = "red", pch = 19, add = TRUE)
+      points3D(x = X[tmp], y = Y[tmp], z = Z[tmp], col = "blue", pch = 19, add = TRUE)
+      
+      legend.text <- if (!is.null(legend.text)) legend.text else c(expression(z >= hat(f)(x, y)), expression(z < hat(f)(x, y)))
+      legend("topright",
+             legend = legend.text,
+             col = c("red", "blue"),
+             pch = 19,
+             bg = 'white',
+             bty = "n")
+      
+    }
+    
+  } else {
+    stop("Type not recognized")
+  }
+  
+  x$Plotinfo <- results
+  invisible(x)
+}
 
 ################################################################################
 ################################################################################
